package TiVo::Slice::Record;

use strict;

our $VERSION =
    (split / /, q$Id: Record.pm,v 1.3 2001/12/03 09:36:59 grant Exp $)[2];

sub new {
    my $class = shift;
    my ($id, $version) = @_;

    die "arguments must be id and version" unless ($id and $version);

    my $self = {};
    bless $self, ref($class) || $class;

    $self->id($id);
    $self->version($version);

    $self->{fields} = [];

    return $self;
}

# fields!
# each record type has different fields, so nothing is defined here
# the record class must define a 'field_defs' subroutine that returns a hash
# it should be like this: (day => [0x44, 'Number'], ...)

sub METHOD::ABSTRACT {
    my $self = shift;
    my $object_class = ref($self);
    my ($file, $line, $method) = (caller(1))[1..3];
    my $loc = "at $file, line $line\n";
    die "someone used an (abstract) TiVo::Slice::Record object $loc"
        if $object_class eq 'TiVo::Slice::Record';
    die "call to abstract method ${method} $loc";
}

sub field_defs { ABSTRACT METHOD @_ }

sub fieldcode {
    my ($self, $name) = @_;
    my %fields = $self->field_defs;
    return $fields{$name}->[0];
}

sub fieldtype {
    my ($self, $name) = @_;
    my %fields = $self->field_defs;
    return $fields{$name}->[1];
}

sub valid_fields {
    my $self = shift;
    my %fields = $self->field_defs;
    return sort keys %fields;
}

sub add {
    my $self = shift;
    my @fields = $self->_new_fields(@_);
    push @{$self->{fields}}, @fields;
}

sub get {
    my $self = shift;
    my $name = shift;
    my @values = grep { $_ == $self->fieldcode($name) } @{$self->{fields}};
    return wantarray ? @values : $values[0];
}

sub _new_fields {
    my $self = shift;
    my %newfields = @_;
    my @fields;
    while (my ($name, $value) = each %newfields) {
        my $type = $self->fieldtype($name);
        die "$name is not a valid field" unless $type;
        my $fieldpkg = "TiVo::Slice::Field::$type";
        eval "require $fieldpkg" or die $@;
        my $field = $fieldpkg->new($value);
        $field->code($self->fieldcode($name));
        push @fields, $field;
    }
    return wantarray ? @fields : $fields[0];
}
            
sub _code {
    my $self = shift;
    if (my $newcode = shift) {
        $self->{code} = pack("n", $newcode);
    }
    return $self->{code};
}

sub id {
    my $self = shift;
    if (my $newid = shift) {
        $self->{id} = $newid;
    }
    return $self->{id};
}

sub version {
    my $self = shift;
    if (my $newversion = shift) {
        $self->{version} = $newversion;
    }
    return $self->{version};
}

sub value {
    my $self = shift;
    my $record = join('',
                      $self->_code,
                      TiVo::Slice::Field::Number->new($self->id)->value,
                      TiVo::Slice::Field::Number->new($self->version)->value,
                      $self->_fields,
                      pack("C", 0x00),
                     );
    return pack("N", length($record)) . $record;
}

sub _fields {
    my $self = shift;
    return $self->_pack_fields(@{$self->{fields}});
}

sub _pack_fields {
    my $self = shift;
    return map { $_->code . $_->value } @_;
}

1;
